"use strict";
/**
 * GoCardless supports webhooks, allowing you to receive real-time notifications
 * when things happen in your account, so you can take automatic actions in
 * response, for example:
 *
 *   When a customer cancels their mandate with the bank, suspend their club membership
 *
 * The `parse` function does two things; firstly, it validates that a webhook you receive
 * is genuinely from GoCardless, and secondly it parses each `event` object in the
 * JSON object into an `GoCardless.Event` class.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = parse;
exports.verifySignature = verifySignature;
exports.InvalidSignatureError = InvalidSignatureError;
const crypto_1 = __importDefault(require("crypto"));
function InvalidSignatureError() {
    this.message = 'The signature header secret does not match your webhook secret!';
    this.name = 'InvalidSignatureError';
}
/**
 * Validates that a webhook was genuinely sent by GoCardless, then parses each `event`
 * object into an array of `GoCardless.Event` classes.
 *
 * @body The raw webhook body.
 * @webhookSecret The webhook endpoint secret for your webhook endpoint, as
 *   configured in your GoCardless Dashboard.
 * @signatureHeader The signature included in the webhook request, as specified
 *   by the `Webhook-Signature` header.
 */
function parse(body, webhookSecret, signatureHeader) {
    verifySignature(body, webhookSecret, signatureHeader);
    const bodyString = typeof body === 'string' ? body : body.toString();
    const eventsData = JSON.parse(bodyString);
    return eventsData.events;
}
/**
 * Validate the signature header. Note, we're using the `crypto.timingSafeEqual`
 * library for the hash comparison, to protect against timing attacks.
 *
 * @body The raw webhook body.
 * @webhookSecret The webhook endpoint secret for your webhook endpoint, as
 *   configured in your GoCardless Dashboard.
 * @signatureHeader The signature included in the webhook request, as specified
 *   by the `Webhook-Signature` header.
 */
function verifySignature(body, webhookSecret, signatureHeader) {
    const bufferDigest = crypto_1.default.createHmac('sha256', webhookSecret).update(body).digest();
    const bufferSignatureHeader = Buffer.from(signatureHeader, 'hex');
    if (bufferDigest.length !== bufferSignatureHeader.length ||
        !crypto_1.default.timingSafeEqual(bufferDigest, bufferSignatureHeader)) {
        throw new InvalidSignatureError();
    }
}
//# sourceMappingURL=webhooks.js.map